#include "Mesh.h"

Mesh::Mesh(void) {
	for(int i = 0; i < MAX_BUFFER; ++i) {
		bufferObject[i] = 0;
	}
	glGenVertexArrays(1, &arrayObject);

	numVertices = 0;
	vertices = NULL;
	colours = NULL;
	type = GL_TRIANGLES;
}

Mesh::~Mesh(void) {
	glDeleteVertexArrays(1, &arrayObject);
	glDeleteBuffers(MAX_BUFFER, bufferObject);
	delete [] vertices;
	delete [] colours;
}

Mesh* Mesh::GenerateTriangle() {
	Mesh * m = new Mesh ();
	m->numVertices = 3;

	m->vertices = new Vector3[m->numVertices];
	m->vertices[0] = Vector3(0.0f, 0.5f, 0.0f);
	m->vertices[1] = Vector3(0.5f, -0.5f, 0.0f);
	m->vertices[2] = Vector3(-0.5f, -0.5f, 0.0f);

	m->colours = new Vector4[m->numVertices];
	m->colours[0] = Vector4(1.0f, 0.0f, 0.0f, 1.0f);
	m->colours[1] = Vector4(0.0f, 1.0f, 0.0f, 1.0f);
	m->colours[2] = Vector4(0.0f, 0.0f, 1.0f, 1.0f);

	m->BufferData();

	return m;
}

void Mesh::BufferData() {
	glBindVertexArray(arrayObject); //OpenGL function don't take object parameters, instead just operate on whatever object is binded
	
	//Put data into graphics memory - Vertex data
	glGenBuffers(1, &bufferObject[VERTEX_BUFFER]); //creates a buffer object and stores its name in  the first index of the buffer object array
	glBindBuffer(GL_ARRAY_BUFFER, bufferObject[VERTEX_BUFFER]); //bind the buffer object we just created
	glBufferData(GL_ARRAY_BUFFER, numVertices * sizeof(Vector3), //copys buffer object to graphics memory
		vertices, GL_STATIC_DRAW); //1st param - size of data to copy, 2nd param - pointer to start of data, 3rd param - how the data will be used (dynamically updated or static)
	glVertexAttribPointer(VERTEX_BUFFER, 3, GL_FLOAT, GL_FALSE, 0, 0); //edits how the vertex's can be accesed, it tells us we have 3 floats per vertex
	glEnableVertexAttribArray(VERTEX_BUFFER); //enables the glVertexAttribArray

	if(colours) {
		//Put data into graphics memory - Colour data
		glGenBuffers(1, &bufferObject[COLOUR_BUFFER]);
		glBindBuffer(GL_ARRAY_BUFFER, bufferObject[VERTEX_BUFFER]);
		glBufferData(GL_ARRAY_BUFFER, numVertices * sizeof(Vector4),
			colours, GL_STATIC_DRAW);
		glVertexAttribPointer(COLOUR_BUFFER, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(COLOUR_BUFFER);
	}

	//not compulsary, but stops another function accidently editting this
	glBindVertexArray(0); //set binded object to NULL
}

void Mesh::Draw() {
	glBindVertexArray(arrayObject); //bind arrayObject for draw Function
	glDrawArrays(type, 0 , numVertices); //using type(GL_TRIANGLES), start at index 0  and draw the numVertices(3)
	glBindVertexArray(0); //unbind arrayObject
}